﻿using System;
using Android.App;
using Android.Content;
using Android.Graphics;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Android.OS;
using Android.Provider;
using Java.IO;
using Uri = Android.Net.Uri;
using Environment = Android.OS.Environment;
using Android.Util;
using Android.Media;

namespace android
{
	[Activity (Label = "android", MainLauncher = true, Icon = "@drawable/icon")]
	public class MainActivity : Activity
	{
		public const int REQUEST_IMAGE_CAPTURE = 1;
		public const string ANDROID_ACTIVITY_LOG = "AA_LOG";
		private const string current_image_uri = "current_image_uri";

		protected override void OnCreate (Bundle bundle)
		{
			base.OnCreate (bundle);
			SetContentView (Resource.Layout.Main);
			TakePictureButton.Click += delegate {
				TakePicture ();
			};
		}

		protected override void OnResume ()
		{
			base.OnResume ();
			if (CurrentImageUri != null) {
				LoadImage (CurrentImageUri);
			}
		}

		protected override void OnPause ()
		{
			base.OnPause ();
			// PictureResult.SetImageBitmap (null);
		}

		protected override void OnSaveInstanceState (Bundle outState)
		{
			base.OnSaveInstanceState (outState);
			if (CurrentImageUri != null) {		
				outState.PutParcelable (current_image_uri, CurrentImageUri);
			}
		}

		protected override void OnRestoreInstanceState (Bundle savedInstanceState)
		{
			base.OnRestoreInstanceState (savedInstanceState);
			if (savedInstanceState.ContainsKey (current_image_uri)) {
				if (savedInstanceState.GetParcelable (current_image_uri) != null) {
					CurrentImageUri = savedInstanceState.GetParcelable (current_image_uri) as Uri;
				}
			}
		}

		public void LoadImage (Uri uri)
		{
			PictureResult.Post (() => {
				var uriPath = uri.Path;
				var dimens = new int[] { PictureResult.Width, PictureResult.Height };

				BitmapFactory.Options opts = new BitmapFactory.Options {
					InJustDecodeBounds = true
				};

				BitmapFactory.DecodeFile (uriPath, opts);
				float scaleFactor = Math.Min (opts.OutWidth / dimens [0], opts.OutHeight / dimens [1]);
				opts.InJustDecodeBounds = false;
				opts.InSampleSize = (int)scaleFactor;
				int rotationOfImage = ImageUtils.RotationOfImage (uriPath);
				Matrix rotationMatrix = new Matrix ();
				rotationMatrix.PreRotate (rotationOfImage);
				using (var bm = BitmapFactory.DecodeFile (uriPath, opts)) {
					using (var rotated = Bitmap.CreateBitmap (bm, 0, 0, bm.Width, bm.Height, rotationMatrix, true)) {
						PictureResult.SetImageBitmap (rotated);
					}
				}
			});
		}

		public void TakePicture ()
		{
			Intent picIntent = new Intent (MediaStore.ActionImageCapture);
			if (picIntent.ResolveActivity (PackageManager) != null) {
				Uri picOutputUri = CreateImageFilename ();
				picIntent.PutExtra (MediaStore.ExtraOutput, picOutputUri);
				StartActivityForResult (picIntent, REQUEST_IMAGE_CAPTURE);
			}
		}

		private Uri CreateImageFilename ()
		{
			// IMG_20150524_110014.jpg
			var timestamp = DateTime.Now.ToString ("yyyyMMdd_HHmmss");
			var imagePrefix = "IMG_" + timestamp;
			File imageFile = new File (ApplicationContext.GetExternalFilesDir (Environment.DirectoryPictures), imagePrefix + ".jpg");
			imageFile.CreateNewFile ();
			CurrentImageUri = Uri.FromFile (imageFile);
			return CurrentImageUri;
		}

		private ImageView PictureResult { get { return FindViewById<ImageView> (Resource.Id.ImgV_TakePictureResult); } }

		private Button TakePictureButton { get { return FindViewById<Button> (Resource.Id.Btn_TakePictureButton); } }

		private Uri CurrentImageUri { get; set; }
	}
}
